import 'dart:async';
import 'dart:convert';

import '../base/common.dart';
import '../base/utils.dart';
import '../device.dart';
import '../doctor.dart';
import '../globals.dart';
import '../runner/mpcli_command.dart';
import '../simple_device_entity.dart';

class DevicesCommand extends MpcliCommand {
  @override
  final String name = 'devices';

  @override
  final String description = 'List all connected devices.';

  @override
  Future<MpcliCommandResult> runCommand() async {
    if (!doctor.canListAnything) {
      throwToolExit(
        "Unable to locate a development device; please run 'mgpcli doctor' for "
        'information about installing additional components.',
        exitCode: 1);
    }

    final List<Device> devices = await deviceManager.getAllConnectedDevices().toList();

    if (devices.isEmpty) {
      printStatus(
        'No devices detected.\n\n'
        "Run 'mgpcli emulators' to list and start any available device emulators.\n\n"
        'Or, if you expected your device to be detected, please run "mgpcli doctor" to diagnose '
        'potential issues, or visit https://flutter.dev/setup/ for troubleshooting tips.');
      final List<String> diagnostics = await deviceManager.getDeviceDiagnostics();
      if (diagnostics.isNotEmpty) {
        printStatus('');
        for (String diagnostic in diagnostics) {
          printStatus('• $diagnostic', hangingIndent: 2);
        }
      }
    } else {
      printStatus('${devices.length} connected ${pluralize('device', devices.length)}:\n');
      await Device.printDevices(devices);
    }
    _printCustomDevice(devices);
    return null;
  }

  void _printCustomDevice(List<Device> device) {
    SimpleDeviceEntity entity = SimpleDeviceEntity();
    List<SimpleDevice> sDevices = [];
    if (device.isNotEmpty) {
      device.forEach((d) {
        SimpleDevice sd = SimpleDevice(
            category: d.category.value,
            deviceId: d.id,
            deviceName: d.name,
            platform: d.platformType.value);
        sDevices.add(sd);
      });
      entity.devices = sDevices;
    }
    String jsonString = jsonEncode(entity.toJson());
    printStatus("__device_start__${jsonString}__device_end__");
  }
}
