
import 'dart:convert';
import '../src/base/file_system.dart';

import '../src/android/android_sdk.dart';
import '../src/base/utils.dart';
import '../src/cache.dart';
import '../src/base/process.dart';
import '../src/globals.dart';


final String cliVersion = "0.1.51";

enum CliEnvironment {
  debug,
  release
}

CliEnvironment cliEnvironment(){
  return CliEnvironment.release;
}


Future<void> writeContent2Path(String path,String fileName, String content) async {
  var file = fs.file('$path/$fileName');

  var sink = file.openWrite();
  sink.write(content);
  await sink.flush();
  await sink.close();
  return Future.value();
}


void writeLocalProperties(File properties) {
  final SettingsFile settings = SettingsFile();
  if (androidSdk != null) {

    settings.values['sdk.dir'] = escapePath(androidSdk.directory);
    settings.values['flutter.sdk'] = escapePath(Cache.flutterRoot);
  }

  settings.writeContents(properties);
}


void cliUnzip(File file, String targetPath) {

  runSync(<String>['unzip', '-o', '-q', file.path, '-d', targetPath]);
}


/*
* cli打印信息
* logString
* */
void cliPrint(String logString){

  cliPrints(logString,true);
}

/*
* cli打印信息
* logString
* isReleasePrint release版本是否打印信息
* */
void cliPrints(String logString, bool isReleasePrint){
  if (cliEnvironment() == CliEnvironment.debug) {
    print(logString);
  } else {

    if (isReleasePrint){
      printStatus(logString);
    }
  }
}



void replaceFilesText(File file, String replaceValue, String newValue) async {

  if (!file.existsSync()) {
    return;
  }

  String fileContent = file.readAsStringSync();

  String fileText = fileContent;

  if (fileContent.contains(replaceValue)) {

    fileContent.replaceAll(replaceValue, newValue);

    fileText.replaceAll(replaceValue, newValue);

    print(fileText);
    print("----------------\n\n");

    file = await file.writeAsString(fileContent);

  }

}
