import 'dart:convert';

import 'base_controller.dart';
import '../response_bean.dart';
import 'package:jaguar/http/context/context.dart';
import '../magpie_attach.dart' as magpie_attach_tool;

final API_DEBUG_ATTACH = '/api/debug/attach';
final API_DEBUG_REFRESH = '/api/debug/refresh';
final API_DEBUG_RELOAD = '/api/debug/reload';

enum DebugActionType { attach, refresh, reload }

class DebugModuleController extends BaseController {
  route(String path, Context context) async {
    ResponseBean bean = ResponseBean('', code: 0);
    if (match(context.uri.path, API_DEBUG_ATTACH)) {
      String directoryStr = context.query.get('directory');
      String targetFileStr = context.query.get('target');
      String deviceStr = context.query.get('device');
      await magpie_attach_tool.magpieAttach(
          ['-d${directoryStr}', '-t${targetFileStr}', '-e${deviceStr}']
              .toList(), (success) {
        bean = handleCallBack(success, DebugActionType.attach);
      });
    } else if (match(context.uri.path, API_DEBUG_RELOAD)) {
      String directoryStr = context.query.get('directory');
      await magpie_attach_tool
          .magpieReload(['-d${directoryStr}', '-mreload'].toList(), (success) {
        bean = handleCallBack(success, DebugActionType.reload);
      });
    } else if (match(context.uri.path, API_DEBUG_REFRESH)) {
      String directoryStr = context.query.get('directory');
      await magpie_attach_tool
          .magpieReload(['-d${directoryStr}', '-mrefresh'].toList(), (success) {
        bean = handleCallBack(success, DebugActionType.refresh);
      });
    }
    return bean;
  }

  ResponseBean handleCallBack(bool success, DebugActionType actionType) {
    String message = '';
    String state = '0';
    switch (actionType) {
      case DebugActionType.attach:
        message = 'attach';
        break;
      case DebugActionType.reload:
        message = 'reload';
        break;
      case DebugActionType.refresh:
        message = 'refresh';
        break;
      default:
        break;
    }
    if (success) {
      state = '0';
      message = '${message}成功';
    } else {
      state = '1';
      message = '${message}失败';
    }
    return ResponseBean(<String, String>{
      jsonEncode('state'): jsonEncode(state),
      jsonEncode('message'): jsonEncode(message)
    });
  }
}
