import 'dart:html';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:workflow/utils/net_util.dart';
import 'dart:convert';

class BaseInfoPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _BaseInfoPageState();
}

class _BaseInfoPageState extends State<BaseInfoPage> {
  List<BaseInfo> _items = [];

  @override
  void initState() {
    super.initState();
    _fetchDoctorInfo();
  }

  //创建item
  Widget _buildItem(String title, String content) {
    return Container(
      padding: EdgeInsets.fromLTRB(20, 20, 20, 20),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Text(
            title,
            style: TextStyle(
              color: Colors.white,
              fontSize: 40.0,
              fontWeight: FontWeight.w300,
            ),
            textAlign: TextAlign.left,
          ),
          Padding(
            padding: EdgeInsets.fromLTRB(0, 8, 0, 8),
            child: Text(
              content ?? "",
              style: TextStyle(
                color: Colors.white,
                fontSize: 15.0,
                letterSpacing: 1,
                fontWeight: FontWeight.w100,
              ),
            ),
          ),
        ],
      ),
    );
  }

  _fetchDoctorInfo() async {
    try {
      String resJson = await NetUtils.get('/api/baseinfo/info');
      Map map = json.decode(resJson);
      List list = map['data'];
      List items = list.map((m) => BaseInfo.fromJson(m)).toList();
      if (mounted) {
        setState(() {
          _items = items;
        });
      }
    } on Exception catch (e) {
      print(e.toString());
    }
  }

  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: ListView.builder(
          itemBuilder: (_, index){
            var info = _items[index];
            return _buildItem(info.title, info.content);
          },
          itemCount: _items.length,
      )
    );
  }
}


class BaseInfo {
  String title;
  String content;

  BaseInfo.fromJson(Map<String, dynamic> json)
      : title = json['title'], content = json['content'];
}