import 'dart:html';
import 'package:flutter/material.dart';
import 'package:workflow/utils/date_format_util.dart';
import 'drop_down_menu.dart';
import 'log_utils.dart';

//日志界面
class LogPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _LogPageState();
}

class _LogPageState extends State<LogPage> {
  String _filterLevel;
  ScrollController _controller = ScrollController();
  
  @override
  void initState() {
    super.initState();
    logManager.onLogMsgUpdate = (){
      setState(() {});
    };
  }

  @override
  void dispose() {
    super.dispose();
    logManager.onLogMsgUpdate = null;
  }

  Widget _header(BuildContext context) {
    return Container(
      height: 60,
      color: Color.fromARGB(0xFF, 0x22, 0x22, 0x22),
      padding: const EdgeInsets.symmetric(horizontal: 10),
      child: Row(
        children: <Widget>[
          Icon(Icons.view_agenda, color: Colors.white),
          Container(width: 10),
          Text('LOGGING', style: TextStyle(color: Colors.white)),
          Expanded(child: Text('')),
          DropDownMenu(
            title: 'level',
            menuList: [
              '全部',
              WBLevel.INFO,
              WBLevel.ERROR,
              WBLevel.SUCCESS,
              WBLevel.WARNING,
              WBLevel.SERVER
            ],
            titleTextStyle: TextStyle(color: Colors.white),
            onSelected: (String s) {
              setState(() {
                _filterLevel = s;
              });
            },
          ),
          IconButton(
            icon: Icon(Icons.delete_forever, color: Colors.white),
            onPressed: () {
              setState(() {
                logManager.removeAllLog();
              });
            },
          )
        ],
      ),
    );
  }

  Widget _logBody(BuildContext context) {
    var filters = <WBLogRecord>[];
    if (_filterLevel == null || _filterLevel == '全部') {
      filters = logManager.logList;
    } else {
      filters = logManager.logList
          .where((item) => item.level == _filterLevel)
          .toList();
      print(filters);
    }

    return Expanded(
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 10),
        child: ListView.builder(
          controller: _controller,
          itemCount: filters.length,
          itemBuilder: (context, index) {
            return Container(
                margin: const EdgeInsets.only(top: 20),
                child: _logItem(context, filters[index]));
          },
        ),
      ),
    );
  }

  Widget _logItem(BuildContext context, WBLogRecord model) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Text(model.level, style: getTextStyle(model.level)),
            Container(height: 3),
            Text(DateFormatter.stringWithDateString(format: 'HH:mm:ss', dateStr: model.time), style: TextStyle(color: Colors.white30))
          ],
        ),
        Container(width: 20),
        Expanded(
          child: Text(model.message, style: getTextStyle(model.level), maxLines: 99),
        )
      ],
    );
  }

  TextStyle getTextStyle(String level) {
    if (level == WBLevel.INFO) {
      return TextStyle(color: Colors.white, fontSize: 13);
    } else if (level == WBLevel.ERROR) {
      return TextStyle(color: Colors.red, fontSize: 13);
    } else if (level == WBLevel.SUCCESS) {
      return TextStyle(color: Colors.green, fontSize: 13);
    } else if (level == WBLevel.WARNING) {
      return TextStyle(color: Colors.yellow, fontSize: 13);
    } else {
      return TextStyle(color: Colors.white, fontSize: 13);
    }
  }

  @override
  Widget build(BuildContext context) {
    Future.delayed(Duration(milliseconds: 20), () {
      _controller.jumpTo(_controller.position.maxScrollExtent);
    });
    return Column(children: <Widget>[_header(context), _logBody(context)]);
  }
}



