import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:loading/indicator/ball_pulse_indicator.dart';
import 'package:loading/loading.dart';
import '../utils/dialog_util.dart';
import '../utils/net_util.dart';
import '../utils/sp_util.dart';

import 'log_utils.dart';

class ReleaseBuild extends StatefulWidget{

  @override
  State<StatefulWidget> createState() {
    return _ReleaseBuildState();
  }
}

class _ReleaseBuildState extends State<ReleaseBuild>{
  bool isBuilding = false;
  final _tPathController = TextEditingController(); //目标目录

  @override
  void initState() {
    super.initState();
    SpUtil.getString(tPathSpKey).then((onValue){
      if(onValue != null){
        setState(() {
          _tPathController.text = onValue;
        });
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Column(children: <Widget>[
      Container(
        padding: EdgeInsets.fromLTRB(20, 10, 20, 10),
        height: 48,
        width: double.infinity,
        child: Row(
          children: <Widget>[
            Text('打包',style: TextStyle(color: Colors.white,fontSize: 18),),
            IconButton(icon: Icon(Icons.help,color: Colors.white,), onPressed: _showHelp,iconSize: 20,padding: EdgeInsets.fromLTRB(10,5,10,5),alignment: Alignment.centerLeft,)
          ],
        ),
        color: Theme.of(context).primaryColor,
      ),
      Container(
        padding: EdgeInsets.fromLTRB(20, 0, 20, 0),
        height: 80,
        child: TextField(
          style: TextStyle(color: Colors.white),
          maxLines: 1,
          controller: _tPathController,
          decoration: _createInputDecoration('请输入编译项目目录', Icon(Icons.folder,color: Theme.of(context).hoverColor,),(){
            setState(() {
              _tPathController.text = '';
            });
          }),
        ),
      ),
      Padding(padding: EdgeInsets.fromLTRB(0, 10, 0, 0)),
      Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Container(
            height: 40,
            width: 300,
            margin: EdgeInsets.all(10),
            child: Material(
              color: isBuilding ? Theme.of(context).splashColor:Theme.of(context).primaryColor,
              textStyle: TextStyle(color: Colors.white),
              child: InkWell(
                child: Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    isBuilding ? Loading(indicator: BallPulseIndicator(), size: 25.0,color: Colors.white):
                    Icon(Icons.build,color: Colors.white,size: 16,),
                    Padding(padding: EdgeInsets.fromLTRB(10, 0, 0, 0),),
                    Text("Android打包"),
                  ],
                ),
                onTap: (){
                  _build(true);
                },
              ),
            ),
          ),
          Container(
            height: 40,
            width: 300,
            margin: EdgeInsets.all(10),
            child: Material(
              color: isBuilding ? Theme.of(context).splashColor:Theme.of(context).primaryColor,
              textStyle: TextStyle(color: Colors.white),
              child: InkWell(
                child: Row(
                  mainAxisSize: MainAxisSize.max,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    isBuilding ? Loading(indicator: BallPulseIndicator(), size: 25.0,color: Colors.white):
                    Icon(Icons.build,color: Colors.white,size: 16,),
                    Padding(padding: EdgeInsets.fromLTRB(10, 0, 0, 0),),
                    Text("Ios打包"),
                  ],
                ),
                onTap: (){
                  _build(false);
                },
              ),
            ),
          ),
        ],
      ),
      Padding(padding: EdgeInsets.fromLTRB(0, 10, 0, 0))
    ],
      crossAxisAlignment: CrossAxisAlignment.start,);
  }

  InputDecoration _createInputDecoration(String hint,Icon icon,Function suffixPress){
    return InputDecoration(labelText: hint,
        labelStyle: TextStyle(color: Colors.blueGrey),
        focusColor: Theme.of(context).hoverColor,
        enabledBorder: UnderlineInputBorder(
            borderSide:BorderSide(color: Colors.white)
        ),
        prefixIcon: icon,
        suffix: IconButton(icon: Icon(Icons.clear,color: Colors.white,size: 18,), onPressed: suffixPress)
    );
  }

  _build(bool isAndroid) async{
    if(_check()) return;

    _savePath();

    setState(() {
      isBuilding = true;
    });
    var param = {'tPath':_tPathController.text};

    var url;
    if(isAndroid){
      url = '/api/release/build_android';
    }else{
      url = '/api/release/build_ios';
    }
    String result = await NetUtils.post(url,param);
    Logger.info('打包结果:$result');
    try {
      Map value = jsonDecode(result);
      if(mounted){
        showInfoDialog(context, '${value['data']} ${value['msg']}');
      }
    } on Exception catch(error) {
      if(mounted){
        showInfoDialog(context, '编译失败');
      }
    }finally{
      if(mounted){
        setState(() {
          isBuilding = false;
        });
      }
    }
  }

  bool _check(){
    if(isBuilding && !mounted) return true;

    return _checkEmpty(_tPathController.text, '编译目标目录不能为空');
  }

  bool _checkEmpty(String content,String tip){
    if(content == null || content.isEmpty){
      showInfoDialog(context, tip);
      return true;
    }
    return false;
  }

  ///保存目录，下次进来自动输入
  _savePath(){
    SpUtil.setString(tPathSpKey, _tPathController.text);
  }

  _showHelp(){
    var helpStr = """1.打包成功后请根据平台发布AAR(Android)，上传编译产物到git(Ios)
2.安卓只支持打包aar，请确保flutter模板通过cli脚手架生成
3.Ios打包时请确保flutter项目中存在lib/main.dart文件
4.打包上传之后安卓可通过gradle依赖，Ios可通过CocoaPods依赖，如需其他方式可通过打包产物手动添加""";
    showInfoDialog(context, helpStr);
  }

}