import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:loading/indicator/ball_pulse_indicator.dart';
import 'package:loading/loading.dart';

import '../utils/dialog_util.dart';
import '../utils/net_util.dart';
import '../utils/sp_util.dart';

import 'log_utils.dart';

///安卓aar发布
class ReleaseUploadAndroid extends StatefulWidget{

  @override
  State<StatefulWidget> createState() {
    return _ReleaseUploadAndroidState();
  }
}

class _ReleaseUploadAndroidState extends State<ReleaseUploadAndroid>{

  final _gitTagVersionController = TextEditingController();
  bool isUploading = false;

  @override
  Widget build(BuildContext context) {
    return Column(children: <Widget>[
      Container(
        margin: EdgeInsets.fromLTRB(10,0,10,0),
        width: double.infinity,
        height: 44,
        child: Align(
          alignment: Alignment.centerLeft,
          child: Row(
            children: <Widget>[
              Text('Android',style: TextStyle(fontSize: 18),),
              IconButton(icon: Icon(Icons.help,color: Colors.white,), onPressed: _showHelp,iconSize: 20,padding: EdgeInsets.fromLTRB(10,5,10,5),alignment: Alignment.centerLeft,)
            ],
          ),
        ),
      ),
      TextField(
        style: TextStyle(color: Colors.white),
        maxLines: 1,
        controller: _gitTagVersionController,
        decoration: _createInputDecoration('Tag Version', Icon(Icons.turned_in_not,color: Theme.of(context).hoverColor,),(){
          setState(() {
            _gitTagVersionController.text = '';
          });
        }),
      ),
      Container(
        height: 40,
        margin: EdgeInsets.fromLTRB(20, 10, 20, 10),
        child: Material(
          color: isUploading ? Theme.of(context).splashColor:Theme.of(context).primaryColor,
          textStyle: TextStyle(color: Colors.white),
          child: InkWell(
            child:Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                isUploading ? Loading(indicator: BallPulseIndicator(), size: 25.0,color: Colors.white):
                Icon(Icons.cloud_upload,color: Colors.white,size: 16,),
                Padding(padding: EdgeInsets.fromLTRB(10, 0, 0, 0),),
                Text("发布AAR"),
              ],
            ),
            onTap: _upload,
          ),
        ),
      ),
    ],
      crossAxisAlignment: CrossAxisAlignment.start,);
  }

  InputDecoration _createInputDecoration(String hint,Icon icon,Function suffixPress){
    return InputDecoration(labelText: hint,
        labelStyle: TextStyle(color: Colors.blueGrey),
        focusColor: Theme.of(context).hoverColor,
        enabledBorder: UnderlineInputBorder(
            borderSide:BorderSide(color: Colors.white)
        ),
        prefixIcon: icon,
        suffix: IconButton(icon: Icon(Icons.clear,color: Colors.white,size: 18,), onPressed: suffixPress)
    );
  }

  _upload() async{
    if(_check()) return;

    var tPath = await SpUtil.getString(tPathSpKey);

    setState(() {
      isUploading = true;
    });
    var param = {'tPath':tPath,'versionTag':_gitTagVersionController.text};

    String result = await NetUtils.post('/api/release/upload_android',param);
    Logger.info('AAR上传结果:$result');
    try {
      Map value = jsonDecode(result);
      if(mounted){
        showInfoDialog(context, '${value['data']} ${value['msg']}');
      }
    } on Exception catch(error) {
      if(mounted){
        showInfoDialog(context, 'AAR上传失败');
      }
    }finally{
      if(mounted){
        setState(() {
          isUploading = false;
        });
      }
    }
  }

  bool _check() {
    if (isUploading && !mounted) {
      return true;
    }
    return _checkEmpty(_gitTagVersionController.text, 'git Tag Version不能为空');
  }

  bool _checkEmpty(String content,String tip){
    if(content == null || content.isEmpty){
      showInfoDialog(context, tip);
      return true;
    }
    return false;
  }

  _showHelp(){
    var helpStr = """1.发布aar之前请先打release aar包
2.发布之前请在模板工程中的根目录gradle.properties中配置maven仓库及发布信息等相关信息
3.发布成之后即可在原生工程中直接通过gardle dependencies直接引用""";
    showInfoDialog(context, helpStr,titleStr: "帮助");
  }

}