import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:loading/indicator/ball_pulse_indicator.dart';
import 'package:loading/loading.dart';

import '../utils/dialog_util.dart';
import '../utils/net_util.dart';
import '../utils/sp_util.dart';
import 'log_utils.dart';

///ios编译产物发布上传
class ReleaseUploadIos extends StatefulWidget{

  @override
  State<StatefulWidget> createState() {
    return _ReleaseUploadIosState();
  }
}

class _ReleaseUploadIosState extends State<ReleaseUploadIos>{
  bool isUploading = false;

  final _gitUrlController = TextEditingController();
  final _gitTagVersionController = TextEditingController();
  final _localGitPathController = TextEditingController();

  final _gitUrlSpKey = 'iosGitUrl';
  final _localGitPath = 'iosLocalGitPath';
  final _tPathSpKey = 'release_tPath';

  @override
  void initState() {
    super.initState();
    SpUtil.getString(_gitUrlSpKey).then((onValue){
      if(onValue != null){
        setState(() {
          _gitUrlController.text = onValue;
        });
      }
    });
    SpUtil.getString(_localGitPath).then((onValue){
      if(onValue != null){
        setState(() {
          _localGitPathController.text = onValue;
        });
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Column(children: <Widget>[
      Container(
        margin: EdgeInsets.fromLTRB(10,0,10,0),
        width: double.infinity,
        height: 48,
        child: Align(
          alignment: Alignment.centerLeft,
          child: Row(
            children: <Widget>[
              Text('iOS',style: TextStyle(fontSize: 18),),
              IconButton(icon: Icon(Icons.help,color: Colors.white,), onPressed: _showHelp,iconSize: 20,padding: EdgeInsets.fromLTRB(10,5,10,5),alignment: Alignment.centerLeft,)
            ],
          ),
        ),
      ),
      Container(
        padding: EdgeInsets.fromLTRB(20, 0, 20, 0),
        height: 80,
        child: TextField(
          style: TextStyle(color: Colors.white),
          maxLines: 1,
          controller: _localGitPathController,
          decoration: _createInputDecoration('请输入本地git仓库目录', Icon(Icons.folder,color: Theme.of(context).hoverColor,),(){
            setState(() {
              _localGitPathController.text = '';
            });
          }),
        ),
      ),
      Container(
        padding: EdgeInsets.fromLTRB(20, 0, 20, 0),
        height: 80,
        child: Row(
          children: <Widget>[
            Expanded(child: TextField(
              style: TextStyle(color: Colors.white),
              maxLines: 1,
              controller: _gitUrlController,
              decoration: _createInputDecoration('请输入需要上传的git地址', Icon(Icons.link,color: Theme.of(context).hoverColor,),(){
                setState(() {
                  _gitUrlController.text = '';
                });
              }),
              ),
              flex: 2,
            ),
            Padding(padding: EdgeInsets.all(10)),
            Expanded(child: TextField(
              style: TextStyle(color: Colors.white),
              maxLines: 1,
              controller: _gitTagVersionController,
              decoration: _createInputDecoration('Tag Version', Icon(Icons.turned_in_not,color: Theme.of(context).hoverColor,),(){
                setState(() {
                  _gitTagVersionController.text = '';
                });
              }),
            ),)
          ],
        )
      ),
      Container(
        height: 40,
        margin: EdgeInsets.fromLTRB(20, 10, 20, 10),
        child: Material(
          color: isUploading ? Theme.of(context).splashColor:Theme.of(context).primaryColor,
          textStyle: TextStyle(color: Colors.white),
          child: InkWell(
            child:Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                isUploading ? Loading(indicator: BallPulseIndicator(), size: 25.0,color: Colors.white):
                Icon(Icons.cloud_upload,color: Colors.white,size: 16,),
                Padding(padding: EdgeInsets.fromLTRB(10, 0, 0, 0),),
                Text("提交到git"),
              ],
            ),
            onTap: _upload,
          ),
        ),
      ),
    ],
      crossAxisAlignment: CrossAxisAlignment.start,);
  }

  InputDecoration _createInputDecoration(String hint,Icon icon,Function suffixPress){
    return InputDecoration(labelText: hint,
        labelStyle: TextStyle(color: Colors.blueGrey),
        focusColor: Theme.of(context).hoverColor,
        enabledBorder: UnderlineInputBorder(
            borderSide:BorderSide(color: Colors.white)
        ),
        prefixIcon: icon,
        suffix: IconButton(icon: Icon(Icons.clear,color: Colors.white,size: 18,), onPressed: suffixPress)
    );
  }

  _upload() async{
    if(_check()) return;

    _savePath();

    var tPath = await SpUtil.getString(_tPathSpKey);

    setState(() {
      isUploading = true;
    });
    var param = {'lPath':_localGitPathController.text,'tPath':tPath,
      'sourceUrl':_gitUrlController.text,'version':_gitTagVersionController.text};

    String result = await NetUtils.post('/api/release/upload_ios',param);
    Logger.info('iOS上传结果:$result');
    try {
      Map value = jsonDecode(result);
      if(mounted){
        showInfoDialog(context, '${value['data']} ${value['msg']}');
      }
    } on Exception catch(error) {
      if(mounted){
        showInfoDialog(context, 'iOS上传失败');
      }
    }finally{
      if(mounted){
        setState(() {
          isUploading = false;
        });
      }
    }
  }

  bool _check(){
    if(isUploading && !mounted) return true;

    return _checkEmpty(_localGitPathController.text, '本地git仓库目录不能为空')
        || _checkEmpty(_gitUrlController.text, 'git地址不能为空')
        || _checkEmpty(_gitTagVersionController.text, 'git Tag Version不能为空');
  }

  bool _checkEmpty(String content,String tip){
    if(content == null || content.isEmpty){
      showInfoDialog(context, tip);
      return true;
    }
    return false;
  }

  ///保存目录，下次进来自动输入
  _savePath(){
    SpUtil.setString(_gitUrlSpKey, _gitUrlController.text);
    SpUtil.setString(_localGitPath, _localGitPathController.text);
  }

  _showHelp(){
    var helpStr = """1.iOS上传之前请先打包
2.iOS发布目前只支持上传到git仓库，仓库创建及后续集成Pods请手动完成""";
    showInfoDialog(context, helpStr,titleStr: "帮助");
  }
}