import 'package:flutter/material.dart';
import 'package:workflow/component/content_panel.dart';
import 'package:workflow/component/device_manager.dart';
import 'package:workflow/component/release_main.dart';

import 'component/debug_module.dart';
import 'component/info/baseinfo_page.dart';
import 'component/log_page.dart';
import 'component/log_utils.dart';
import 'component/sidebar.dart';
import 'utils/net_util.dart';

void main() {
  NetUtils.init('127.0.0.1:8080');
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Workflow',
      theme: ThemeData(
        primarySwatch: Colors.deepPurple, // TODO 主题色待定
        hoverColor: Colors.deepPurpleAccent,
        dividerTheme: DividerThemeData().copyWith(thickness: 1),
      ),
      home: HomePage(title: 'Home'),
    );
  }
}

class HomePage extends StatefulWidget {
  HomePage({Key key, this.title}) : super(key: key);
  final String title;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  final _titles = ['连接设备', 'Debug', 'Release', '日志', '环境信息'];
  final _widgets = [
    DeviceManager(),
    DebugModule(null),
    ReleaseMain(),
    LogPage(),
    BaseInfoPage(),
  ];
  int currentIndex = 0;

  @override
  void initState() {
    super.initState();
    logManager.connectSocketClient();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Row(
        children: <Widget>[
          Sidebar(
            menus: _titles,
            onTap: (index) => setState(() {
              currentIndex = index;
              Logger.info('点击了${_titles[index]}');
            }),
          ),
          ContentPanel(
            title: _titles[currentIndex],
            builder: (context) => _widgets[currentIndex],
            hideBar: currentIndex == 3, //日志有个性化的头部，避免出现两个
            darkTheme: true, //默认暗黑模式，如果页面确实白色，可以关闭
          )
        ],
      ),
    );
  }
}
